#
# Copyright (c) 2008-2016 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphFactorySettings
from pyams_content.component.theme.interfaces import IThemesManagerTarget
from pyams_content.shared.blog.interfaces import IBlogManager, BLOG_CONTENT_TYPE, IBlogFolder
from pyams_portal.interfaces import IPortalContext
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.component.interfaces import ISite
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectRemovedEvent

# import packages
from pyams_content.shared.blog import BlogPost
from pyams_content.shared.common.manager import SharedTool
from pyams_skin.skin import UserSkinnableContent
from pyams_utils.traversing import get_parent
from pyramid.events import subscriber
from zope.container.folder import Folder
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


@implementer(IBlogFolder)
class BlogFolder(Folder):
    """Blog folder class"""

    def is_deletable(self):
        for element in self.values():
            if not element.is_deletable():
                return False
        return True


@implementer(IBlogManager, IParagraphFactorySettings, IThemesManagerTarget, IAttributeAnnotatable, IPortalContext)
class BlogManager(SharedTool, UserSkinnableContent):
    """Nlog manager class"""

    folder_factory = BlogFolder

    shared_content_type = BLOG_CONTENT_TYPE
    shared_content_factory = BlogPost

    allowed_paragraphs = FieldProperty(IParagraphFactorySettings['allowed_paragraphs'])
    auto_created_paragraphs = FieldProperty(IParagraphFactorySettings['auto_created_paragraphs'])

    sequence_name = ''  # use default sequence generator
    sequence_prefix = ''

    def is_deletable(self):
        for element in self.values():
            if not element.is_deletable():
                return False
        return True


@subscriber(IObjectAddedEvent, context_selector=IBlogManager)
def handle_added_blog_manager(event):
    """Register blog manager when added"""
    site = get_parent(event.newParent, ISite)
    registry = site.getSiteManager()
    if registry is not None:
        registry.registerUtility(event.object, IBlogManager, name=event.object.__name__)


@subscriber(IObjectRemovedEvent, context_selector=IBlogManager)
def handle_deleted_blog_manager(event):
    """Un-register blog manager when deleted"""
    site = get_parent(event.oldParent, ISite)
    registry = site.getSiteManager()
    if registry is not None:
        registry.unregisterUtility(event.object, IBlogManager, name=event.object.__name__)
